import * as react_jsx_runtime from 'react/jsx-runtime';
import { QueryKey, FetchInfiniteQueryOptions } from '@tanstack/react-query';

/**
 * A component that allows you to use `usePrefetchInfiniteQuery` in JSX, avoiding the limitations of React hooks.
 * @see {@link https://suspensive.org/docs/react-query/PrefetchInfiniteQuery Suspensive Docs}
 * @example
 * ```tsx
 * <PrefetchInfiniteQuery queryKey={['queryKey']} queryFn={queryFn} />
 * ```
 */
declare function PrefetchInfiniteQuery<TQueryFnData = unknown, TError = unknown, TData = TQueryFnData, TQueryKey extends QueryKey = QueryKey>(options: FetchInfiniteQueryOptions<TQueryFnData, TError, TData, TQueryKey>): react_jsx_runtime.JSX.Element;

export { PrefetchInfiniteQuery };
