import * as react_jsx_runtime from 'react/jsx-runtime';
import { QueryKey } from '@tanstack/react-query';
import { ReactNode } from 'react';
import { UseSuspenseInfiniteQueryOptions, UseSuspenseInfiniteQueryResult } from './useSuspenseInfiniteQuery.cjs';
import './OmitKeyof-CFsZLI4k.cjs';

/**
 * We provide these components to clearly express what causes suspense at the same depth.
 * `<SuspenseInfiniteQuery/>` serves to make `useSuspenseInfiniteQuery` easier to use in jsx.
 * @see {@link https://suspensive.org/docs/react-query/SuspenseInfiniteQuery Suspensive Docs}
 * @example
 * ```tsx
 * import { SuspenseInfiniteQuery } from '@suspensive/react-query'
 *
 * // You can use infiniteQueryOptions as props.
 * <SuspenseInfiniteQuery {...infiniteQueryOptions()}>
 *   {({ data, fetchNextPage }) => {
 *     return <></>
 *   }}
 * </SuspenseInfiniteQuery>
 * ```
 */
declare const SuspenseInfiniteQuery: <TQueryFnData = unknown, TError = unknown, TData = TQueryFnData, TQueryKey extends QueryKey = QueryKey>({ children, ...options }: UseSuspenseInfiniteQueryOptions<TQueryFnData, TError, TData, TQueryKey> & {
    children: (query: UseSuspenseInfiniteQueryResult<TData, TError>) => ReactNode;
}) => react_jsx_runtime.JSX.Element;

export { SuspenseInfiniteQuery };
