import * as react_jsx_runtime from 'react/jsx-runtime';
import { ReactNode } from 'react';
import { SuspenseQueriesOptions, SuspenseQueriesResults } from './useSuspenseQueries.cjs';
import '@tanstack/react-query';
import './useSuspenseQuery.cjs';
import './OmitKeyof-CFsZLI4k.cjs';

/**
 * We provide these components to clearly express what causes suspense at the same depth.
 * `<SuspenseQueries/>` serves to make `useSuspenseQueries` easier to use in jsx.
 * @see {@link https://suspensive.org/docs/react-query/SuspenseQueries Suspensive Docs}
 * @example
 * ```tsx
 * import { SuspenseQueries } from '@suspensive/react-query'
 *
 * <SuspenseQueries queries={[firstQueryOptions(), secondQueryOptions()]}>
 *   {([{ data: firstQueryData }, { data: secondQueryData }]) => {
 *     return <></>
 *   }}
 * </SuspenseQueries>
 * ```
 */
declare function SuspenseQueries<T extends any[]>({ children, queries, }: {
    queries: readonly [...SuspenseQueriesOptions<T>];
    children: (queries: SuspenseQueriesResults<T>) => ReactNode;
}): react_jsx_runtime.JSX.Element;

export { SuspenseQueries };
