import * as react_jsx_runtime from 'react/jsx-runtime';
import { QueryKey } from '@tanstack/react-query';
import { ReactNode } from 'react';
import { UseSuspenseQueryOptions, UseSuspenseQueryResult } from './useSuspenseQuery.js';
import './OmitKeyof-CFsZLI4k.js';

/**
 * We provide these components to clearly express what causes suspense at the same depth.
 * `<SuspenseQuery/>` serves to make `useSuspenseQuery` easier to use in jsx.
 * @see {@link https://suspensive.org/docs/react-query/SuspenseQuery Suspensive Docs}
 * @example
 * ```tsx
 * import { SuspenseQuery } from '@suspensive/react-query'
 *
 * // You can use QueryOptions as props.
 * <SuspenseQuery {...queryOptions()}>
 *   {({ data, isLoading }) => {
 *     return <></>
 *   }
 * </SuspenseQuery>
 * ```
 */
declare const SuspenseQuery: <TQueryFnData = unknown, TError = unknown, TData = TQueryFnData, TQueryKey extends QueryKey = QueryKey>({ children, ...options }: UseSuspenseQueryOptions<TQueryFnData, TError, TData, TQueryKey> & {
    children: (queryResult: UseSuspenseQueryResult<TData, TError>) => ReactNode;
}) => react_jsx_runtime.JSX.Element;

export { SuspenseQuery };
