import { QueryKey, UseInfiniteQueryOptions, UseInfiniteQueryResult, InfiniteData } from '@tanstack/react-query';
import { O as OmitKeyof } from './OmitKeyof-CFsZLI4k.cjs';

interface UseSuspenseInfiniteQueryResult<TData = unknown, TError = unknown> extends OmitKeyof<UseInfiniteQueryResult<TData, TError>, keyof Pick<UseInfiniteQueryResult<TData, TError>, 'isPlaceholderData'>> {
    data: InfiniteData<TData>;
    status: 'success';
}
type UseSuspenseInfiniteQueryOptions<TQueryFnData = unknown, TError = unknown, TData = TQueryFnData, TQueryKey extends QueryKey = QueryKey> = OmitKeyof<UseInfiniteQueryOptions<TQueryFnData, TError, TData, TQueryFnData, TQueryKey>, 'suspense' | 'useErrorBoundary' | 'enabled' | 'placeholderData' | 'networkMode'>;
/**
 * This hook is wrapping `useInfiniteQuery` of `@tanstack/react-query` v4 with default suspense option.
 * @see {@link https://suspensive.org/docs/react-query/useSuspenseInfiniteQuery Suspensive Docs}
 */
declare function useSuspenseInfiniteQuery<TQueryFnData = unknown, TError = unknown, TData = TQueryFnData, TQueryKey extends QueryKey = QueryKey>(options: UseSuspenseInfiniteQueryOptions<TQueryFnData, TError, TData, TQueryKey>): UseSuspenseInfiniteQueryResult<TData, TError>;

export { type UseSuspenseInfiniteQueryOptions, type UseSuspenseInfiniteQueryResult, useSuspenseInfiniteQuery };
